package it.unimi.dsi.webgraph;


/*		 
 * Copyright (C) 2003, 2004, 2005 Sebastiano Vigna 
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

import gnu.getopt.Getopt;
import gnu.getopt.LongOpt;
import it.unimi.dsi.fastutil.ints.IntArrayList;
import it.unimi.dsi.fastutil.ints.IntIterator;
import it.unimi.dsi.fastutil.io.FastBufferedInputStream;
import it.unimi.dsi.mg4j.index.CompressionFlags;
import it.unimi.dsi.mg4j.util.ProgressMeter;

import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Arrays;

/** The main method of this class allows one to permute the nodes of a graph.
 * The permutation must be specified using a list of integers in {@link java.io.DataInput} format (as many as the nodes).
 * Note <var>i</var> in the graph will become the node specified by the
 * <var>i</var>-th element of the list.
 * 
 * <strong>Warning</strong>: this class was actually permuting a graph using the <em>inverse</em> of the provided
 * permutation&mdash;node <var>i</var> of the new graph would have come from the <var>i</var>-th element of the list.
 * 
 * @deprecated Use {@link it.unimi.dsi.webgraph.Transform#map(ImmutableGraph, int[])} instead.
 */

public class PermuteBVGraph implements CompressionFlags {

	private PermuteBVGraph() {}

	private final static boolean ASSERTS = true;
	static { if ( ASSERTS ); }

	/** The offset step used in loading the source graph. */
	private static int offsetStep = 1;

	/** Whether we should invert the given permutation. */
	private static boolean invert;

	private static void permuteBVGraph( String sourceBasename, String destBasename, String permFile ) throws IOException {
		int i, x;
		
		ProgressMeter pm = new ProgressMeter();
		// Load the source BVGraph without actually loading it into memory.
		final BVGraph source = (BVGraph)BVGraph.load( sourceBasename, offsetStep, pm );

		System.err.println( "Reading permutation..." );

		final int[] s2d = new int[ source.numNodes() ], d2s = new int[ source.numNodes() ];
		final DataInputStream dis = new DataInputStream( new FastBufferedInputStream( new FileInputStream( permFile ), 1024 * 1024 ) );
		if ( invert ) {
			for( i = 0 ; i < s2d.length; i++ ) {
				x = dis.readInt();
				s2d[ x ] = i;
				d2s[ i ] = x;
			} 
		} 
		else {
			for( i = 0 ; i < s2d.length; i++ ) {
				x = dis.readInt();
				s2d[ i ] = x;
				d2s[ x ] = i;
			}
		} 

		dis.close();

		if ( ASSERTS ) {
			boolean[] b = new boolean[ s2d.length ];
			for( i = 0; i < s2d.length; i++ ) {
				assert s2d[ i ] >= 0;
				assert s2d[ i ] < s2d.length;
				assert ! b[ s2d[ i ] ];
				b[ s2d[ i ] ] = true;
			}			 
		} 

		BVGraph.store( new ImmutableGraph() {
			private final IntArrayList succ = new IntArrayList();
			public int numNodes() { return source.numNodes(); }
			public long numArcs() { return source.numArcs(); }
			public int[] successorArray( int x ) {
				succ.clear();
				IntIterator i = source.successors( s2d[ x ] );
				while( i.hasNext() ) succ.add( d2s[ i.nextInt() ] );
				Arrays.sort( succ.elements(), 0, succ.size() );
				return succ.elements();
			}
			public int outdegree( int x ) { return source.outdegree( s2d[ x ] ); }
			},
			destBasename,
			pm
		);
	}

	public static void main( String args[] ) throws IOException {
		int ch;
		LongOpt longopts[] = new LongOpt[] {
			new LongOpt( "help", LongOpt.NO_ARGUMENT, null, 'h' ),
			new LongOpt( "offset-step", LongOpt.REQUIRED_ARGUMENT, null, 's' ),
			new LongOpt( "invert", LongOpt.NO_ARGUMENT, null, 'i' ),
		};

		Getopt g = new Getopt( "TransposeBVGraph", args, "his:", longopts );
		g.setOpterr( true );

		while ( ( ch = g.getopt() ) != -1 ) switch ( ch ) {
		case 'h':
			System.err.println( "Usage: PermuteBVGraph SOURCE DEST PERM" );
			System.err.println( "Permutes the SOURCE BVGraph using the given PERM" );
			System.err.println( "and stores the result into the DEST BVGraph." );
			System.err.println( "" );
			System.err.println( "Optional arguments:" );
			System.err.println( "  -s, --offset-step    step to load offsets (default: " + offsetStep + ")" );
			System.err.println( "  -i, --invert         use the inverse of the given permutation" );
			System.err.println( "Help:" );
			System.err.println( "  -h, --help              print this help screen" );
			System.err.println( "" );
			return;

		case 's':
			offsetStep = Integer.parseInt( g.getOptarg() );
			break;

		case 'i':
			invert = true;
			break;

		case '?':
			return;

		default:
			break;
		}

		if ( args.length - g.getOptind() != 3 ) {
			System.err.println( "Wrong number (" + ( args.length - g.getOptind() ) + ") of arguments." );
			return;
		}
		
		permuteBVGraph( args[ g.getOptind() ], args[ g.getOptind() + 1 ], args[ g.getOptind() + 2 ] );
	}
}

// Local Variables:
// mode: jde
// tab-width: 4
// End:
